from typing import Iterator


def interpret(url: str) -> Iterator[dict]:
    post_id = [x for x in url.split("/") if x][-1]
    yield {
        "fileid": post_id,
        "filename": f"Instagram {post_id}",
        "sourceurl": url,
        "tags": [
            "instagram"
        ]
    }



from typing import Optional

from bs4 import Tag, BeautifulSoup, PageElement
from commmons import get_host_url

from hypersquirrel.util import soup_from_url

MEDIABOOK_ENABLED = False


def get_vkey_href_title(li: Tag):
    vkey: str = li.attrs.get("data-video-vkey")
    href = f"view_video.php?viewkey={vkey}"
    title = None
    a = li.find("a", attrs=dict(href=f"/{href}"))
    if a:
        title = a.attrs.get("title")
    return vkey, href, title


def get_profile_name(soup: BeautifulSoup) -> Optional[str]:
    div = soup.find("div", class_="nameSubscribe")
    if div:
        return div.find("h1").text.strip()


def try_append_thumbnail(li: PageElement, file: dict, attrs: dict):
    img = li.find("img", class_="thumb", attrs=attrs)

    if not img:
        return

    thumbnailurl = img.attrs.get("src")
    if thumbnailurl:
        file["thumbnailurl"] = thumbnailurl

    if MEDIABOOK_ENABLED:
        mediabook = img.attrs.get("data-mediabook")
        if mediabook:
            file["thumbnailvideourl"] = mediabook


def scrape(url):
    soup = soup_from_url(url)
    profile_name = get_profile_name(soup)

    for li in soup.find_all("li", class_="videoblock"):
        def try_append_uploader(title: str) -> str:
            uploader = profile_name
            if not uploader:
                usrdiv = li.find("div", class_="usernameWrap")
                if usrdiv:
                    uploader = usrdiv.find("a").text

            return f"[{uploader}] {title}" if uploader else title

        vkey, href, title = get_vkey_href_title(li)
        if title:
            file = dict(
                fileid=vkey,
                sourceurl=f"{get_host_url(url)}/{href}",
                filename=try_append_uploader(title.strip())
            )

            try_append_thumbnail(li, file, attrs={"alt": title})

            yield file



from commmons import get_host_url

from hypersquirrel.util import soup_from_url


def sanitize(text):
    text = text.replace("\n", " ")
    while "  " in text:
        text = text.replace("  ", " ")

    return text


def scrape_v2ph(url):
    soup = soup_from_url(url)

    for div in soup.find_all("div", class_="thumbnail card"):
        a = div.find("a", class_="media-cover")
        img = div.find("img")
        if img is None:
            continue

        file = {
            "fileid": "v2ph" + a.attrs["href"].split("/")[-1].split(".")[0].split("?")[0],
            "filename": img.attrs["alt"],
            "sourceurl": "vpr://" + get_host_url(url) + a.attrs["href"],
            "thumbnailurl": img.attrs["data-src"]
        }

        media_meta = div.find_next("div", class_="media-meta")
        if media_meta:
            file["filename"] = sanitize(media_meta.text)

        yield file



import os
from dataclasses import dataclass
from typing import Union

import requests
from commmons import head, get_query_params

CHANNELS_API_URL = "https://www.googleapis.com/youtube/v3/channels"
VIDEOS_API_URL = "https://www.googleapis.com/youtube/v3/videos"


@dataclass
class UploadIdFinder:
    USE_API_TO_FIND_UPLOAD_ID = False

    url: str
    apikey: str

    def _get_url_component(self, preceding_component: str) -> Union[str, None]:
        if f"{preceding_component}/" in self.url:
            return head(self.url.split(f"{preceding_component}/")[1].split("/"))
        return None

    def _to_upload_id(self, external_id: str):
        if UploadIdFinder.USE_API_TO_FIND_UPLOAD_ID:
            body = requests.get(CHANNELS_API_URL, params={
                "part": "contentDetails",
                "id": external_id,
                "key": self.apikey
            }).json()

            return body["items"][0]["contentDetails"]["relatedPlaylists"]["uploads"]

        return "UU" + external_id.lstrip("UC")

    def _get_upload_id_by_watch(self) -> str:
        video_id = get_query_params(self.url).get("v") or self._get_url_component("watch")
        body = requests.get(VIDEOS_API_URL, params=dict(
            id=video_id,
            part='id,snippet',
            type='channel',
            key=self.apikey
        )).json()
        channel_id = head(body["items"])["snippet"]["channelId"]
        return self._to_upload_id(channel_id)

    def _get_upload_id_by_channel(self) -> str:
        channel_id = self._get_url_component('channel')
        return self._to_upload_id(channel_id)

    def _get_upload_id_by_user(self) -> str:
        username = self._get_url_component('user')
        body = requests.get(CHANNELS_API_URL, params={
            "part": "id",
            "forUsername": username,
            "key": self.apikey
        }).json()
        external_id = head(body.get("items", [{}]))["id"]
        return self._to_upload_id(external_id)

    def get_upload_id(self) -> str:
        upload_id_map = dict(
            channel=self._get_upload_id_by_channel,
            user=self._get_upload_id_by_user,
            watch=self._get_upload_id_by_watch
        )

        existing_key = head([key for key in upload_id_map.keys() if f"/{key}" in self.url])
        func = upload_id_map[existing_key]
        return func()


def get_playlist_id(url: str) -> Union[str, None]:
    if "/playlist?list=" in url:
        return get_query_params(url).get("list")

    return None


def get_adequate_thumbnail_url(thumbnails: dict) -> Union[str, None]:
    if not thumbnails:
        return None

    if "medium" in thumbnails:
        return thumbnails["medium"]["url"]

    return thumbnails.get(head(list(thumbnails.keys())))


def get_all_videos(upload_id: str, apikey: str):
    url = "https://www.googleapis.com/youtube/v3/playlistItems"
    params = {
        "playlistId": upload_id,
        "key": apikey,
        "part": "snippet",
        "maxResults": 50
    }

    while True:
        body = requests.get(url, params=params).json()

        for item in body["items"]:
            snippet = item["snippet"]
            video_id = snippet["resourceId"]["videoId"]
            thumbnailurl = get_adequate_thumbnail_url(snippet["thumbnails"])
            yield {
                "fileid": video_id,
                "filename": snippet["title"],
                "sourceurl": f"https://youtube.com/watch/{video_id}",
                "thumbnailurl": thumbnailurl
            }

        next_page_token = body.get("nextPageToken")

        if not next_page_token:
            break

        params["pageToken"] = next_page_token


def scrape_youtubeapi(url: str):
    apikey = os.getenv("YOUTUBE_APIKEY")
    playlist_id = get_playlist_id(url)

    if not playlist_id:
        print("Getting upload_id as a substitute")
        playlist_id = UploadIdFinder(url, apikey).get_upload_id()

    yield from get_all_videos(playlist_id, apikey)



from commmons import get_host_url
from lxml import html


def scrape(url):
    def get_body_with_ytdl() -> str:
        from yt_dlp import YoutubeDL
        ytdl = YoutubeDL()
        ie = ytdl.get_info_extractor("SpankBang")
        mobj = ie._match_valid_url(url)
        video_id = mobj.group('id') or mobj.group('id_2')
        return ie._download_webpage(
            url.replace('/%s/embed' % video_id, '/%s/video' % video_id),
            video_id, headers={'Cookie': 'country=US'})

    tree = html.fromstring(get_body_with_ytdl())
    host_url = get_host_url(url)

    divs = tree.xpath("//div[contains(@class, 'video-item')]")
    for div in divs:
        if "data-id" not in div.attrib:
            continue

        dataid = div.attrib["data-id"]
        atags = div.xpath("./a")

        if not atags:
            continue

        imgs = div.xpath("./a/picture/img")
        if not imgs:
            continue

        atag = atags[0]
        img = imgs[0]

        yield {
            "fileid": "sb" + dataid,
            "sourceurl": host_url + atag.attrib["href"],
            "filename": img.attrib["alt"],
            "thumbnailurl": img.attrib["data-src"]
        }



import re
from typing import Iterator, List

from hypersquirrel.core import Watchlist
from hypersquirrel.literalinterpreterfactory import get_interpreter
from hypersquirrel.scraperfactory import get_scraper

PAGED_PATTERN = re.compile("<\d+-\d+>")


def create_watchlist(url: str, max_items: int):
    match = PAGED_PATTERN.findall(url)
    if len(match) == 1:
        pages = match[0].replace("<", "").replace(">", "").split("-")
        return Watchlist(
            url=url.replace(match[0], "${page}"),
            max_items=max_items,
            page_min=int(pages[0]),
            page_max=int(pages[1])
        )
    return Watchlist(
        url=url,
        max_items=max_items
    )


def scrape(w: Watchlist) -> Iterator[dict]:
    return w.scrape(get_scraper(w))


def scrape_literal_urls(urls: List[str]) -> dict:
    for url in set(urls):
        interpreter = get_interpreter(url)
        if interpreter:
            yield from interpreter(url)



import requests
from commmons import html_from_url, merge
from bs4 import BeautifulSoup

_HEADERS = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0",
    "Cache-Control": "no-cache, no-store, must-revalidate",
    "Pragma": "no-cache",
    "Expires": "0"
}


def html_from_url_with_headers(url, referer=None):
    headers = merge(_HEADERS, dict(Referer=referer or url))
    return html_from_url(url, headers=headers)


def soup_from_url(url, **kwargs):
    s = requests.Session()
    r = s.get(url, headers=_HEADERS, **kwargs)
    return BeautifulSoup(r.text, "lxml")



from typing import Iterator

from hypersquirrel.scraper.ehen import _to_post_id


def interpret(url: str) -> Iterator[dict]:
    post_id = _to_post_id(url)
    yield {
        "fileid": post_id,
        "filename": f"ehentai {post_id}",
        "sourceurl": url,
        "tags": [
            "ehentai"
        ]
    }



import re
from typing import Iterator, Callable

from hypersquirrel.literalinterpreter.instagram import interpret as interpret_instagram
from hypersquirrel.literalinterpreter.ehen import interpret as interpret_ehen

interpreters = {
    "^.*instagram.com\/(p|reel)\/[a-zA-z0-9-_]*.$": interpret_instagram,
    "^.*e-hentai\.org\/g\/[0-9]+\/.*$": interpret_ehen,
}


def get_interpreter(url: str) -> Callable[[str], Iterator[dict]]:
    for regex, interpreter in interpreters.items():
        if re.match(regex, url):
            return interpreter
